/*  Input (implementation)
    Copyright (C) 2001  Clinton Ebadi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "input.hh"
#include <iostream> // only need istream, but <istream> doesn't always exist
#include <vector>
#include <algorithm>

/* TODO:
 * more error handling in funcs that set values (check for NULL)
 */

game_parser::game_parser ()
  : handlers()
{
  input = &std::cin;
  output = &std::cout;
  c_del = '\n';
}

void
game_parser::event_handler (std::string key, game_handler_t func)
{
  handlers[key] = func; // todo: maybe inline?
  return;
}

void
game_parser::remove_event_handler (const std::string key)
{
  handlers.erase (key);
  return;
}

void
game_parser::input_stream (std::istream* new_stream)
{
    input = new_stream;
}

void
game_parser::output_stream (std::ostream* new_stream)
{
  output = new_stream;
}

char
game_parser::command_delimeter (char new_del)
{
  
  char temp_del = c_del;
  c_del = new_del;
 
  return temp_del;

}

bool 
game_parser::get_command ()
{
  str_vec temp_vector;
  std::string temp_string;

  *input >> temp_string;

  hmap_t::const_iterator key_name = handlers.find (temp_string);

  if (key_name == handlers.end())
    throw (key_not_found (temp_string));

  temp_vector.push_back (temp_string);

  while (*input && input->peek() != c_del) 
    {
      *input >> temp_string;
      temp_vector.push_back (temp_string);
    }

  game_handler_t temp_handler = handlers[temp_vector[0]];
  bool flag = false;
  if ((temp_vector.size() - 1) >= temp_handler.req_args)
    if (temp_handler.opt_args == -1)
      flag = true;
    else if ((temp_vector.size() - 1) <= (temp_handler.req_args + temp_handler.opt_args))
      flag = true;

  if (!flag)
    throw (not_enough_args (temp_vector[0], temp_handler.args_msg, 
			    (temp_vector.size() - 1), temp_handler.req_args, 
			    (temp_handler.req_args + temp_handler.opt_args)));

  try {
    temp_handler.func (temp_vector,*this);
  } 
  catch (handler_error h) {
    *output << h.message << std::endl;
    if (h.fatal) return false;
  }
  
  return true; 
}

game_parser::not_enough_args::not_enough_args
(std::string ccmd, std::string mmsg, unsigned int ggiven, unsigned int mmin, unsigned int mmax)
  : cmd (ccmd), msg (mmsg), given (ggiven), min (mmin), max (mmax)
{ }

game_parser::key_not_found::key_not_found (std::string kkey)
  : key (kkey)
{ }
