#include "input.hh"
#include <stdlib.h>

bool moo (str_vec, game_parser& p)
{
  std::ostream& out = *p.get_output_stream ();
  out << "Moo Moo, I am the great cow!\n";
  return true;
}

bool f_foo (str_vec f, game_parser& p)
{
  std::ostream& out = *p.get_output_stream ();
  out << "Fear my mighty foo, " << f[1] // only required argument
      << ".\n";

  const char* bar;
  try {
    bar = f.at (2).c_str ();
  } catch (...) { // catches anything
    out << "I'm only good enough for one arg? Fine. I hate you.\n";
    return true; // operation success, even though I hate you
  }

  out << "My optional arg was " << bar;
  return true;
}

bool quit (str_vec, game_parser& p) 
{
  std::ostream& out = *p.get_output_stream ();
  out << "Exiting\n";
  exit (0);
  return true; // will never get here
}

bool print (str_vec args, game_parser& parse)
{
  std::ostream& temp = *parse.get_output_stream ();
  temp << args[1] << std::endl;
  return true;
}

bool get_input (str_vec, game_parser& parse)
{
  std::istream& tin = *parse.get_input_stream ();
  std::ostream& tout = *parse.get_output_stream ();
  tout << "Enter your name: ";
  std::string tstring;
  tin >> tstring;
  tout << "Your name is: " << tstring << std::endl;
  return true;
}

int main () {

  game_parser foo;
  foo.event_handler ("moo", 
		     game_handler_t (&moo, "Too much moo!",0,0));
  // notice how the number of required and optional args can be left out if both are 0
  foo.event_handler ("quit", 
		     game_handler_t (&quit, "I ONLY NEED ONE ARG!"));
  // if opt is 0, it can be left out
  foo.event_handler ("print", 
		     game_handler_t (&print, "Print what?",1));
  foo.event_handler ("name", 
		     game_handler_t (&get_input, "Give me your name later"));
  // foo takes one required argument, and one optional for a total of two maximum
  foo.event_handler ("foo",
		     game_handler_t (&f_foo, "Give me at least one argument, please?", 1, 1));

  while (true) {
    try {
      std::cout << std::endl << ">>> ";
      foo.get_command ();
    } 
    catch (game_parser::not_enough_args a) {
      std::cerr 
	<< "Error: Wrong number of arguments given to " << a.cmd 
	<< "! Gave: " << a.given << " Minumum: " << a.min << " Max: "
	<< a.max << std::endl
	<< a.cmd << " gave this message: " << a.msg;
    }
    catch (game_parser::key_not_found k) {
      std::cerr << "Error: key (" << k.key << ") not found\n";
    }
  }

  std::cout << std::endl;
  return 0;

}
